rm(list=ls())
# Pakiet do zapisu danych w formacie zoo
require(zoo)
require(ggplot2)
require(rugarch)

source("LoadFundData.R")
# source("LoadWigData.R")

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")


##########################################################
# Ustawienia do analizy                                  #
##########################################################

T  <- length(r)                        # liczba obserwacji dla pelnej proby
N  <- 1250                             # ustalamy probe na podstawie ktorej liczymy statystyki
r  <- tail(r,N)                        # stopy zwrotu
R  <- coredata(r)

##########################################################
# Statystyki opisowe dla logarytmicznych stp zwrotu WIG #
##########################################################

mu = sum(R)/N

R0    <- R - mu
M2    <- sum(R0^2)/N
M3    <- sum(R0^3)/N
M4    <- sum(R0^4)/N

sig <- sqrt(M2)       ## zmiennosc
S   <- M3/(sig^3)     ## skosnosc
K   <- M4/(sig^4)     ## kurtoza

# to samo z pakietem moments
require(moments)
M1   = moment(R, order = 1, central = FALSE, na.rm = TRUE)
M2   = moment(R, order = 2, central = TRUE, na.rm = TRUE)
M3   = moment(R, order = 3, central = TRUE, na.rm = TRUE)
M4   = moment(R, order = 4, central = TRUE, na.rm = TRUE)
mu0  = M1
sig0 = sqrt(M2)
S0   = M3/(sig0^3)
K0   = M4/(sig0^4)

# Annualizacja
Nyear <- 250
muA   <- mu*Nyear
sigA  <- sig*sqrt(Nyear)


##################################
# Test na normalnosc stop zwrotu #
##################################

#D'Agostino test for skewness
agostino.test(R)

#Anscombe-Glynn test of kurtosis for normal samples
anscombe.test(R)

#Jarque-Bera test of normality
jarque.test(R)

#######################################################
# Rozkad t-Studenta                                  #
#######################################################

x = seq(-5,5,0.01)
junk <- cbind(x,dnorm(x),ddist("std",x,shape=10),ddist("std",x,shape=5),ddist("std",x,shape=3))
colnames(junk) <- c("x","vInf","v10","v5","v3")

ggplot(data = data.frame(junk), aes(x = x)) + 
  geom_line(aes(y = vInf,colour="v=Inf")) + 
  geom_line(aes(y = v10,colour="v=10")) +
  geom_line(aes(y = v5,colour="v=5")) +
  geom_line(aes(y = v3,colour="v=3")) +
  labs(title="rozkad t-Studenta", y="", x="", caption="")+
  theme_bw()+
  scale_colour_manual("", 
                      breaks = c("v=Inf", "v=10","v=5","v=3"),
                      values = c("black", "red", "blue", "green"))


#######################################################
# Empiryczna funkcja gestosci (kernel density plot)   #
#######################################################

R0        <- (R-mu)/sig
bwdth     <- 0.1

ggplot(data.frame(R0), aes(x = R0)) +
  theme_bw() +
  geom_histogram(binwidth = bwdth, colour = "white", fill = "yellow4", size = 0.1) +
  stat_function(fun = function(x) dnorm(x)*N*bwdth, color = "red", size = 1)                        # rozklad normalny
  # + stat_function(fun = function(x) ddist("std",x,shape=5)*N*bwdth, color = "black", size = 1)    # rozklad t-Studenta
  # + xlim(-7,-2)


#######################################################
# Wykres kwantyl-kwantyl (QQplot)                     #
#######################################################

R0     <- (R-mu)/sig

# Tabela kwantyli
q              <- seq(0.01, 0.20, 0.01)
QQtable        <- data.frame(quantile(R0,q),qnorm(q))   # rozklad normalny
names(QQtable) <- c("kwantyl empiryczny", "kwantyl teoretyczny")

# QQ plot
q            <- seq(0.001, 0.999, 0.001)
Qemp         <- qnorm(q)    
#Qemp         <- quantile(rnorm(N),q)
#Qemp         <- quantile(R0,q)                    # kwantyl empiryczny
Qteo         <- qnorm(q)                          # kwantyl teoretyczny (rozklad normalny)
#v=5;   Qteo  <- qt(q,v)*sqrt((v-2)/v)             # rozklad t-Studenta (wariancja to v/v-2)
lim0    <- c(-5,5)                           # zakres na wykresie
par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(Qemp,Qteo, main="QQplot", col="red", xlim = lim0, ylim = lim0,
     xlab="kwantyl empiryczny", ylab="kwantyl teoretyczny") # plots the results
abline(a=0,b=1, lwd=2)


#######################################################
# Estymacja parametrow  rozkladu t-Studenta           #
#######################################################

# Metoda momentow (K - kurtoza)
v0 <- 4 + 6/(K-3)

# Metoda najwikszej wiarygodnoci
require(MASS)

d0 <- fitdistr(R0, "normal")
d0$loglik

d1 <- fitdistr(R0, "t", m = 0, start = list(s=sqrt((v0-2)/v0), df=v0), lower=c(0.001,3))
d1$loglik
v=d1$estimate[[2]]

# Comparison
(d1$loglik-d0$loglik)/N

# QQ plot
q        <- seq(0.001, 0.999, 0.001)
Qemp     <- quantile(R0,q)                     # kwantyl empiryczny
Qteo     <- qt(q,v)*sqrt((v-2)/v)              # rozklad t-Studenta (wariancja to v/v-2)
lim0    <- c(-5,5)                             # zakres na wykresie
par(mfrow=c(1,1), cex = 0.7, bty="l")
plot(Qemp,Qteo, main="QQplot", col="red", xlim = lim0, ylim = lim0,
     xlab="kwantyl empiryczny", ylab="kwantyl teoretyczny") # plots the results
abline(a=0,b=1, lwd=2)

#######################################################
# Autokorelacje, czyli zaleznosci w czasie            #
#######################################################

require(tseries)

# ACF dla stop zwrotu
z0 <- acf(R,20, plot=TRUE)
plot(z0$acf[2:20], type="h", main="ACF dla stop zwrotu", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.2), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))

# ACF dla kwadratow stop zwrotu
z1 <- acf(R^2,20, plot=FALSE)
plot(z1$acf[2:20], type="h", main="ACF dla kwadratow stop zwrotu", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.4), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))

# Kros-korelacje dla stop zwrotu i kwaratow stop zwrotu
z2 <- ccf(R^2, R, 20, type="correlation", plot=TRUE)
plot(z2$acf[22:40], type="h", main="korelacja kwadratow stop zwrotu i opoznionych st. zwrotu", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.2), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))

plot(z2$acf[20:1], type="h", main="korelacja st. zwrotu i opoznionych kwadratow st. zwrotu", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.2), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))

# panel wykresow
par(mfrow=c(2,2), cex = 0.7, bty="l")
plot(z0$acf[2:20], type="h", main="ACF dla stop zwrotu", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.2), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))
plot(z1$acf[2:20], type="h", main="ACF dla kwadratow stop zwrotu", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.4), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))
plot(z2$acf[22:40], type="h", main="korelacja kw. st. zwrotu i op. st. zw.", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.2), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))
plot(z2$acf[20:1], type="h", main="korelacja st. zw. i op. kw. st. zw.", xlab="Lag", ylab="ACF", ylim=c(-0.2,0.2), las=1)
abline(h=c(2/sqrt(length(R)),0,-2/sqrt(length(R))),lty=c(2,1,2))


# testy Ljunga-Boxa

Box.test(R, lag = 20, type = c("Ljung-Box"))
Box.test(R^2, lag = 20, type = c("Ljung-Box"))




