rm(list = ls())
source("MRFzR_FunkcjeBlok2.R")
require(zoo)
require(rugarch)
require(car)

source("LoadFundData.R")
#source("LoadWigData.R")

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##########################################################
# Ustawienia do modelu VaR                               #
##########################################################


T   <- length(r)                  # liczba obserwacji
p   <- 0.01                       # poziom tolerancji VaR
N   <- 1250                       # ustalamy dlugosc proby (5lat)
r   <- tail(r,N)                  # dane
backN    <- 250                   # dlugosc danych do backtestu
w_length <- 1000                  # szerokosc okna estymacji


###############################################################
# Metoda swiatel (Bazylea II); w oryginale dla p=0.01         #
###############################################################

## Symulacja historyczna, schemat rolowany
varHS <- rollapply(r, width=w_length,
                   function(w){quantile(w,p)},
                   by=1, align="right")


varHS <- lag(varHS, -1)           # przesuwamy indeksy aby uwzglednic, ze VaR(t+1) = quantile(t)
rr    <- r[index(varHS)]          # zrealizowane zwroty, rr - realized returns
etaHS  <- tail(rr<=varHS, backN)  # szereg przekroczen (VaR violations)

# empiryczny udzial przekroczen (powinien byc rowny poziomowi tolerancji p)
nHS  <- sum(etaHS); nHS      # l. przekroczen
piHS <- mean(etaHS); piHS    # udzial przekroczen

VaRplot(alpha=p, actual=tail(rr,backN), VaR=tail(varHS,backN))
title(main="Symulacja historyczna: przekroczenia VaR")


## Rozklady parametryczne: normalny i t-Studenta

# rolowana srednia i odchylenie standardowe
MAmean  <- rollapply(r, width=w_length, mean, by=1, align="right")
MAstd   <- rollapply(r, width=w_length, sd,   by=1, align="right")

# Wartosci VaR
varN <- MAmean + MAstd*qdist("norm", p);         varN <- lag(varN, -1)  
# Przekroczenia VaR
etaN  <- tail(rr<=varN, backN)
# Udzialy / liczba przekroczen
nN = sum(etaN); piN = mean(etaN)
# wykres
VaRplot(alpha=p, actual=tail(rr,backN), VaR=tail(varN,backN))
title(main="Rozklad normalny: przekroczenia VaR")

# t-Student
varT <- MAmean + MAstd*qdist("std", p, shape=5); varT <- lag(varT, -1)
etaT  <- tail(rr<=varT, backN)
nT = sum(etaT); piT = mean(etaT)
VaRplot(alpha=p, actual=tail(rr,backN), VaR=tail(varT,backN))
title(main="Rozklad t-Studenta: przekroczenia VaR")

# Powroc do linii 29 i ustaw dluzszy okres do backtestingu 
backN    <- 1000                        
w_length <- 250
# Co sie zmienilo?

## Sekwencyjny backtesting (liczba prekroczen w oknie 250)
## Aby zrozumiec co sie dzieje trzeba zapoznac sie z "metoda swiatel"
## Ta czesc ma sens jedynie dla ustawien z linii 72-73


eta <- etaHS # etaN, etaT
var <- varHS # varN, varT

#rolowana liczba przekroczen w oknie 250 obserwacji
empEx <- rollapply(eta, width=250, sum, align="right")

par(mfrow=c(2,1),oma=c(1,1,1,1),mar=c(1,1,1,1))
plot(c(zoo(NA, head(tail(index(eta), backN), 249)), empEx), xlab="",ylab="", xaxt="n", main="liczba przekroczen w oknie 250 obs.")
abline(h=4,col="green",lty=2)
abline(h=9,col="orange",lty=2)
# rolowana metoda swiatel
TRplot(actual=tail(rr,backN), VaR=tail(var,backN))

# Wyjasnienie metody swiatel
# Rozklad dwumianowy
require(binom)
require(knitr)
p       <- 0.01     # poziom tolerancji
backN   <- 250      # dlugosc danych do backtestu
lex     <- 0:12     # liczba przekroczen

Table = cbind(dbinom(lex,backN,p),pbinom(lex,backN,p))
colnames(Table) = c("pdf","cdf"); rownames(Table) = lex
kable(Table,digits=3)







