rm(list=ls())
# Pakiet do zapisu danych w formacie zoo
require(zoo)
require(ggplot2)
require(rugarch)
require(forecast)
require(binom)
require(knitr)
require(car)

source("LoadFundData.R")
# source("LoadWigData.R")
source("MRFzR_FunkcjeBlok2.R")

# Panel wykresow: pelna proba
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##################################
# Ustawienia do modelu VaR       #
##################################
T   <- length(r)                  # liczba obserwacji
p   <- 0.05                       # poziom tolerancji VaR
N   <- 1250                       # ustalamy dlugosc proby (5lat)
r   <- tail(r,N)                  # dane

backN    <- 250                   # dlugosc danych do backtestu
w_length <- 1000                  # szerokosc okna estymacji

## Obliczenie VaR, realizacji oraz przekrocze VaR. Symulacja historyczna 
varHS  <- rollapply(r, width=w_length,
                   function(w){quantile(w,p)},
                   by=1, align="right")
varHS  <- lag(varHS, -1)           # przesuwamy indeksy aby uwzglednic, ze VaR(t+1) = quantile(t)
rr     <- r[index(varHS)]          # zrealizowane zwroty, rr - realized returns
etaHS  <- tail(rr<=varHS, backN)  # szereg przekroczen (VaR violations)
nHS    <- sum(etaHS); nHS      # l. przekroczen
piHS   <- mean(etaHS); piHS    # udzial przekroczen

dev.off()
VaRplot(alpha=p, actual=tail(rr,backN), VaR=tail(varHS,backN))
title(main="Symulacja historyczna: przekroczenia VaR")

# podsatwiamy obliczenia dla wybranej metody

var = varHS
eta = etaHS
pi  = piHS
n1  = nHS

###############################
#    1. Testy regresji        #
###############################
c(pi,p)
Acf(as.numeric(coredata(eta)))

# 1a. Pokrycie. Dla eta[t] = pi + eps[t] H0: pi = p
reg1a <- lm(eta~1)
summary(reg1a)
linearHypothesis(reg1a, hypothesis.matrix=1, rhs=p, test="Chisq") # H0: pi = p

# 1b. Niezaleznosc. Dla eta[t] = pi + rho*eta[t-1] + eps[t] H0: rho = 0  
reg1b <- lm(eta[-1]~lag(eta,-1))
summary(reg1b)
linearHypothesis(reg1b, hypothesis.matrix=c(0,1), rhs=0, test="Chisq")

# 1c. Niezalezne pokrycie. Dla eta[t] = pi + rho*eta[t-1] + eps[t]   H0: pi = p, rho = 0  
reg1c <- lm(eta[-1]~lag(eta,-1))
summary(reg1c)
linearHypothesis(reg1c, hypothesis.matrix=diag(1,2), rhs=c(p,0), test="Chisq")

######################################################
# 2. Test pokrycia, rozklad dwumianowy               #
#    H0: n1 ~ Binom(n,p), 95% przedzial ufnosci      #
######################################################

# 95% przedzia ufnosci
kable(cbind(seq(0,20,2),pbinom(seq(0,20,2),backN,p)),col.names = c("n1","cdf"),digits=3)

LB = qbinom(0.025,backN,p)
UB = qbinom(0.975,backN,p)
print(paste0("LB=",LB,"; UB=",UB,", l. przekroczen =",n1))

# Test Kupca
n  <- length(eta); n
n1 <- sum(eta);    n1
n0 <- n - n1;      n0
pi <- n1/n;        pi

# statystyka testu
LR_uc   = (p/pi)^n1 *((1-p)/(1-pi))^n0
stat_uc = -2*log(LR_uc)
prob_uc = 1 - pchisq(q=stat_uc,df=1, lower.tail=TRUE)
paste0("Statystyka testu Kupca: ", round(stat_uc,3),"; p-value:", round(prob_uc,3))

# z pakietem rugarch
temp <- VaRTest(alpha=p, actual=rr, VaR=var)
paste0("Statystyka testu Kupca: ", round(temp$uc.LRstat,3),"; p-value:", round(temp$uc.LRp,3))


###########################################################
# 3. Test niezaleznosci (Christofersen independence test) #
#    H0: p(eta[t]=1|eta[t-1]=1) = p(eta[t]=1|eta[t-1]=0)  #
###########################################################
Acf(as.numeric(coredata(tail(eta,backN))))  # funkcja autokorelacji 
## test Ljung-Boxa, H0: niezaleznosc
Box.test(as.numeric(coredata(tail(eta, backN))), 1, type="Ljung-Box")


# Christofersen independence test
eta1 <- coredata(eta[-length(eta)])
eta0 <- coredata(eta[-1])
n00 = sum(!eta1 & !eta0) # no exceedance after no exceedance
n01 = sum(!eta1 &  eta0)  # exceedance after no exceedance
n10 = sum( eta1 & !eta0)  # no exceedance after exceedance
n11 = sum( eta1 &  eta0)   # exceedance after exceedance

#n0  = n00 + n10
#n1  = n01 + n11

pi0 = n01 / (n00+n01) # prawdopodobienstwo przekroczenia po braku przekroczenia
pi1 = n11 / (n10+n11) # prawdopodobienstwo przekroczenia po przekroczeniu
pi  = (n01+n11) / (n00+n01+n10+n11)

c(pi0,pi1,pi)

#LR_ind   = pi^n1*(1-pi)^n0 / (pi0^n01 * pi1^n11 * (1-pi0)^n00  * (1-pi1)^n10)
LR_ind   = (pi/pi0)^n01 * (pi/pi1)^n11 * ((1-pi)/(1-pi0))^n00 * ((1-pi)/(1-pi1))^n10
stat_ind = -2*log(LR_ind)
prob_ind = 1 - pchisq(stat_ind,1)

paste0("Statystyka testu Ch1: ", round(stat_ind,3),"; p-value:", round(prob_ind,3))


##############################################################
# 4. Test niezaleznego pokrycia                              #
# Christofersen conditional coverage test                    #
#    H0: p(eta[t]=1|eta[t-1]=1) = p(eta[t]=1|eta[t-1]=0) = p #
##############################################################

# Christofersen conditional coverage test
LR_cc   = p^n1*(1-p)^n0 / (pi0^n01 * pi1^n11 * (1-pi0)^n00  * (1-pi1)^n10)
stat_cc = -2*log(LR_cc)
prob_cc = 1 - pchisq(stat_cc,1)
prob_cc

paste0("Statystyka testu Ch2: ", round(stat_cc,3),"; p-value:", round(prob_cc,3))

# stat_cc = stat_uc + stat_ind
c(stat_uc+stat_ind,stat_cc)

# z pakietem rugarch
temp <- VaRTest(alpha=p, actual=coredata(rr), VaR=coredata(var))
paste0("Statystyka testu Ch2: ", round(temp$cc.LRstat,3),"; p-value:", round(temp$cc.LRp,3))


##############################
# Modele parametryczne       #
##############################

MAmean  <- rollapply(r, width=w_length, mean, by=1, align="right")
MAstd   <- rollapply(r, width=w_length, sd,   by=1, align="right")
var     <- MAmean + MAstd*qdist("norm", p);    
# var     <- MAmean + MAstd*qdist("std", p, shape=5); 
var     <- lag(var, -1)

dev.off()
VaRplot(alpha=p, actual=rr, VaR=var)
title(main="Model parametryczny: przekroczenia VaR")

temp <- VaRTest(alpha=p, actual=coredata(rr), VaR=coredata(var))
paste0("Statystyka testu Kupca: ", temp$uc.LRstat,"; p-value:", temp$uc.LRp)
paste0("Statystyka testu Ch2: ", temp$cc.LRstat,"; p-value:", temp$cc.LRp)

#################
# Model  EWMA   #
#################

lambda <- 0.94
## specyfikacja modelu
EWMAspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                        variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                        fixed.pars=list(alpha1=1-lambda, omega=0), distribution.model = "norm")
                        #fixed.pars=list(alpha1=1-lambda, omega=0,shape=5), distribution.model = "std")

## rolowany VaR
varEWMA <- rollapply(r, width=w_length,
                        function(w) {
                            # model nie wymaga estymacji, od razu prognozujemy
                            # na podstawie specyfikacji
                            frc <- ugarchforecast(EWMAspec,data=w, n.ahead=1)
                            quantile(frc, p)
                        },
                        by=1, align="right")
# przesuwamy indeksy aby wskazywaly na dni prognozy
varEWMA <- lag(varEWMA, -1)

VaRplot(alpha=p, actual=rr, VaR=varEWMA)
title(main="EWMA: przekroczenia VaR")

temp <- VaRTest(alpha=p, actual=coredata(rr), VaR=coredata(varEWMA))
paste0("Statystyka testu Kupca: ", temp$uc.LRstat,"; p-value:", temp$uc.LRp)
paste0("Statystyka testu Ch2: ", temp$cc.LRstat,"; p-value:", temp$cc.LRp)


##################
# Model  GARCH   #
##################

GARCHspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                         variance.model=list(model="sGARCH", garchOrder=c(1,1)),
                         distribution.model = "norm")
                         # fixed.pars=list(shape=5), distribution.model = "std")

varGARCH <- rollapply(r, width=w_length,
                        function(w) {
                            fit <- ugarchfit(data=w, spec=GARCHspec, solver="hybrid")
                            frc <- ugarchforecast(fit, n.ahead=1)
                            quantile(frc, p)
                        },
                        by=1, align="right")
varGARCH <- lag(varGARCH, -1)

VaRplot(alpha=p, actual=rr, VaR=varGARCH)
title(main="GARCH: przekroczenia VaR")

temp <- VaRTest(alpha=p, actual=coredata(rr), VaR=coredata(varGARCH))
paste0("Statystyka testu Kupca: ", temp$uc.LRstat,"; p-value:", temp$uc.LRp)
paste0("Statystyka testu Ch2: ", temp$cc.LRstat,"; p-value:", temp$cc.LRp)

#############################################
# Backtesing z ugarchroll z pakietu rugarch #
#############################################

step = 10; # how often we reestimate the GARCH model

varGARCHRoll <- ugarchroll(spec=GARCHspec, data=r, refit.every=step, forecast.length=backN, refit.window="moving", window.size=w_length, calculate.VaR=TRUE, VaR.alpha=c(0.01, 0.025, 0.05))
report(varGARCHRoll, VaR.alpha = 0.05)
report(varGARCHRoll, VaR.alpha = 0.01)


#######################
# Dodatek: moc testow #
#######################

backN    <- 250                   # dlugosc danych do backtestu
alpha    <- 0.05                  # poziom istotnosci (przedzial ufnosci 1-alpha)

###############################
# Moc Testow. Metoda swiatel  #
###############################

nexceed <- list(0:4,5:9,10:backN)
pval    <- 0.01*c(1,2,3,5)

tab1     <- matrix(0, nrow=4, ncol=3)
colnames(tab1) <- c("n1<=4", "4<n<=9", "9<n"); rownames(tab1) <- paste0("p=",pval)
for(j in 1:4) {
  for(i in 1:3) {
    tab1[j,i] <- round(sum(dbinom(nexceed[[i]], backN, pval[j])), 3)
  }
}
tab1

###################################
# Moc Testow. Rozklad dwumianowy  #
###################################

# przedzial nieodrzucenia H0 dla rozkladu dwumianowego, funkcja norejInt
norejInt(p=0.05, n=backN, alpha=alpha)
pbinom(18,backN,0.05) - pbinom(3,backN,0.05)

# przedzial nieodrzucenia H0 dla
pval <- 0.01*c(1,2,3,5)
nval <- c(250, 500, 1000, 1500)

tab2 <- matrix(NA,nrow=length(pval), ncol=2*length(nval))
colnames(tab2)  <- paste0("n=",c(rbind(nval, nval)))
rownames(tab2)  <- paste0("p=",pval)

for(i in 1:length(nval)){
  for(j in 1:length(pval)){
    tab2[j,c(2*i-1, 2*i)] <- norejInt(pval[j], nval[i], alpha)
  }}
tab2

# Moc testu
p      = 0.01   # poziom tolerancji z H0
pTrue  = 0.02   # poziom tolerancji dla DGP

temp <- norejInt(p=p, n=backN, alpha=alpha)  # obszar nieodrzucenia H0
temp[1]:temp[2]
moc = 1-sum(dbinom(temp[1]:temp[2], backN, pTrue))
paste0("Moc testu dla p=",p,", gdy prawdiwe p wynosi:",pTrue," wynosi:", round(moc,3))

###################
# Moc testu Kupca #
###################

p      = 0.02  # poziom tolerancji z H0
pTrue  = 0.02   # poziom tolerancji dla DGP

# wartosc LR_uc
LR_uc <- function(n, n1, p) {
  n0 <- n - n1
  pi <- n1/n
  (p/pi)^n1 *((1-p)/(1-pi))^n0
}

# obszar nieodrzucenia H0
nexceed = 0:backN
temp <- nexceed[(-2*log(LR_uc(backN,nexceed,p)) <= qchisq(1-alpha,1))]
temp
# moc jako prawdopodoienstwo znalezienia sie w obszarze nieodrzucen warunkowo wzgledem pTrue
moc = 1-sum(dbinom(temp, backN, pTrue))
paste0("Moc testu dla p=",p,", gdy prawdiwe p wynosi:",pTrue," wynosi:", round(moc,3))

