rm(list = ls())
require(zoo)
require(rugarch)
require(car)
source("MRFzR_FunkcjeBlok2.R")

source("LoadFundData.R")
# source("LoadWigData.R")

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##########################################################
# Ustawienia do modelu VaR                               #
##########################################################

T   <- length(r)                  # liczba obserwacji
p   <- 0.05                       # poziom tolerancji VaR
N   <- 1250                       # ustalamy dlugosc proby (5lat)
r   <- tail(r,N)                  # dane
backN    <- 250                   # dlugosc danych do backtestu
w_length <- 1000                  # szerokosc okna estymacji
rr       <- tail(r,backN)

################################################
### Backtesting dla ES - test McNeila i Freya  #
################################################

## Symulacja historyczna, schemat rolowany
##########################################
varHS <- rollapply(r, width=w_length,
                   function(w){quantile(w,p)},
                   by=1, align="right")

esHS  <- rollapply(r, width=w_length,
                  function(w){mean(w[w < quantile(w,p)])},
                  by=1, align="right")

varHS <- lag(varHS, -1)           # przesuwamy indeksy aby uwzglednic, ze VaR(t+1) = quantile(t)
esHS  <- lag(esHS, -1)

rr     <- r[index(varHS)]          # zrealizowane zwroty, rr - realized returns
etaHS  <- tail(rr<=varHS, backN)  # szereg przekroczen (VaR violations)

# testy Kupca i Cristoffersena dla VaR
VaRTest(alpha=p, actual=coredata(rr), VaR=coredata(varHS))

# test McNeila i Freya
mean(varHS[etaHS])
mean(esHS[etaHS])
mean(rr[etaHS])
ESVaRplot(alpha=p, actual=rr, VaR=varHS, ES=esHS); title(main="Symulacja historyczna")

temp      <- rr[etaHS]-esHS[etaHS]
stat_MF   <- mean(temp)/ (sd(temp)/sqrt(length(temp)))
prob_MF   <- 1- pnorm(abs(stat_MF))
prob_MF 

# to samo w pakiecie rugarch
ESTest(alpha=p, actual=coredata(rr), ES=coredata(esHS), VaR=coredata(varHS))

##########################
## modele parametryczne  #
##########################

# rolowana srednia i odchylenie standardowe
MAmean  <- rollapply(r, width=w_length, mean, by=1, align="right")
MAstd   <- rollapply(r, width=w_length, sd,   by=1, align="right")

# rozklad normalny
# Wartosci VaR
varN <- MAmean + MAstd*qdist("norm", p);          
# Przekroczenia VaR
etaN <- tail(rr<=varN, backN)
# Wartosci ES
qf   <- function(x) qdist("norm", p=x)
esN  <- MAmean + MAstd*(1/p * integrate(qf, 0, p)$value)

varN <- lag(varN, -1)           
esN  <- lag(esN, -1)
etaN <- tail(rr<=varN, backN)  

ESVaRplot(alpha=p, actual=rr, VaR=varN, ES=esN); title(main="Rozklad normalny")
ESTest(alpha=p, actual=coredata(rr), ES=coredata(esN), VaR=coredata(varN))

# rozklad t-studenta 
# Wartosci VaR
varT <- MAmean + MAstd*qdist("std", p, shape=5)         
# Przekroczenia VaR
etaT <- tail(rr<=varT, backN)
# Wartosci ES
qf       <- function(x) qdist("std", p=x, shape=5)
esT  <- MAmean + MAstd*(1/p * integrate(qf, 0, p)$value)

varT <- lag(varT, -1)           
esT  <- lag(esT, -1)
etaT <- tail(rr<=varT, backN)  

ESVaRplot(alpha=p, actual=rr, VaR=varT, ES=esT); title(main="Rozklad t-studenta")
ESTest(alpha=p, actual=coredata(rr), ES=coredata(esT), VaR=coredata(varT))

#################
## Model EWMA   #
#################

lambda <- 0.94

## specyfikacja modelu
EWMAspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=FALSE),
                       variance.model=list(model="iGARCH", garchOrder=c(1,1)),
                       fixed.pars=list(alpha1=1-lambda, omega=0), distribution.model = "norm")
                      #fixed.pars=list(alpha1=1-lambda, omega=0,shape=5), distribution.model = "std")

## funkcja kwantylowa
qf     <- function(x) qdist("norm", p=x)
#qf     <- function(x) qdist("std", p=x, shape=5)
eqt <-  integrate(qf, 0, p)$value

varesEWMA <- rollapply(r, width=w_length,
                        function(w) {
                            frc <- ugarchforecast(EWMAspec,data=w, n.ahead=1)
                            var <- quantile(frc, p)
                            sigma <- sigma(frc)
                            es <- sigma*eqt/p
                            return(c(var, es))
                        },
                        by=1, align="right")
varesEWMA <- lag(varesEWMA, -1)
varEWMA <- varesEWMA[,1]
esEWMA  <- varesEWMA[,2]
etaEWMA <- (rr<varEWMA)

ESVaRplot(alpha=p, actual=rr, VaR=varEWMA, ES=esEWMA); title(main="Model EWMA")
ESTest(alpha=p, actual=coredata(rr), ES=coredata(esEWMA), VaR=coredata(varEWMA))

#################
## Model GARCH  #
#################

## specyfikacja modelu
GARCHspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                       variance.model=list(model="sGARCH", garchOrder=c(1,1)),
                       distribution.model = "norm")
                       #fixed.pars=list(shape=5), distribution.model = "std")

## funkcja kwantylowa
qf     <- function(x) qdist("norm", p=x)
#qf     <- function(x) qdist("std", p=x, shape=5)
eqt <-  integrate(qf, 0, p)$value

varesGARCH <- rollapply(r, width=w_length,
                           function(w) {
                             fit <- ugarchfit(data=w, spec=GARCHspec, solver="hybrid")
                             frc <- ugarchforecast(fit, n.ahead=1)
                             var <- quantile(frc, p)
                             sigma <- sigma(frc)
                             mu <- fitted(frc)
                             es <- mu + sigma*eqt/p
                             return(c(var, es))
                           },
                           by=1, align="right")
varesGARCH <- lag(varesGARCH, -1)
varGARCH <- varesGARCH[,1]
esGARCH  <- varesGARCH[,2]
etaGARCH <- (rr<varGARCH)

ESVaRplot(alpha=p, actual=rr, VaR=varGARCH, ES=esGARCH); title(main="Model GARCH")
ESTest(alpha=p, actual=coredata(rr), ES=coredata(esGARCH), VaR=coredata(varGARCH))


#############################################################
#        Backtesting calego rozkladu/rozmiaru straty        #
#                  test Berkowitza                          #
#############################################################

###########################
# Rozklad parametryczny   #
###########################

MAmean  <- rollapply(r, width=w_length, mean, by=1, align="right"); MAmean=lag(MAmean,-1)
MAstd   <- rollapply(r, width=w_length, sd,   by=1, align="right"); MAstd=lag(MAstd,-1)

## standaryzacja (rstar musi byc z N(0,1))
rstar <- (rr-MAmean)/ MAstd
# PIT~U(0,1)
PIT   <- pdist("norm",rstar)
# PIT   <- pdist("std",rstar,shape=5) # t-Student
rstar <- qnorm(PIT)

regB <- lm(rstar[-1]~lag(rstar,-1))
summary(regB)
linearHypothesis(regB, hypothesis.matrix=diag(1,2), rhs=c(0,0), test="Chisq")

# to samo w pakiecie rugarch (tylko, ze test LR zamiast LM)
testDist <- BerkowitzTest(data = rstar, lags = 1, significance = 0.05)
testDist
testDist$LRp #p-value
testDist$H0  #hipoteza zerowa
testDist$Decision  #decyzja

##########
## GARCH #
##########

GARCHspec <- ugarchspec(mean.model=list(armaOrder=c(0,0), include.mean=TRUE),
                         variance.model=list(model="sGARCH", garchOrder=c(1,1)),
                         fixed.pars=list(shape=5), distribution.model = "norm")
                         # fixed.pars=list(shape=5), distribution.model = "std")

temp <- rollapply(r, width=w_length,
                        function(w) {
                            fit <- ugarchfit(data=w, spec=GARCHspec, solver="hybrid")
                            frc <- ugarchforecast(fit, n.ahead=1)
                            sigma <- sigma(frc)
                            mu <- fitted(frc)
                            return(c(mu, sigma))
                        },
                        by=1, align="right")
# przesuwamy indeksy aby wskazywaly na dni prognozy
temp <- lag(temp, -1)
MAmean = temp[,1]; MAstd = temp[,2];
rstar <- (rr-MAmean)/ MAstd
# PIT~U(0,1)
PIT   <- pdist("norm",rstar)
# PIT   <- pdist("std",rstar,shape=5) # t-Student
rstar <- qnorm(PIT)

BerkowitzTest(data = rstar, lags = 1, significance = 0.05)
