rm(list=ls())
# Pakiet do zapisu danych w formacie zoo
require(zoo)
require(ggplot2)
require(rugarch)
#require(rje)
#require(lubridate)
#require(dplyr)
require(bizdays)
require(xts)
#require(XML)
#require(tools)
#require(timeDate)
#require(RQuantLib)

#source("LoadFundData.R")
source("LoadWigData.R")
ret = r

# Panel wykresow: pelna proba 
par(mfrow=c(2,1), cex = 0.7, bty="l")
plot(P, main="Poziom", xlab="", ylab="")
plot(r, main="stopy zwrotu", xlab="", ylab="")

##########################################################
# Ustawienia do modelu VaR                               #
##########################################################
p   <- 0.025                      # poziom tolerancji VaR
r   <- tail(r,1260)               # ostatnie 5 lat danych
R   <- coredata(r)

###################################################################
#  Wyznaczenie VaR, VEV i MRM wedug Zalacznika II rozporzadzenia #
#  Czyli Cornish-Fisher dla p=2.5% i horyzontu 3 lat              #
###################################################################

Nyear      <- 252               # liczba obserwacji w roku
RHP        <- 3                 # horyzont inwestycji w latach (RHP - recommended holding period)
N          <- RHP * Nyear       # RHP w dniach

# Statystyki
M0 <- length(R)
M1 <- sum(R)/M0          # srednia
R0 <- R - M1             # stopy wokol sredniej 
M2 <- sum(R0^2)/M0
M3 <- sum(R0^3)/M0
M4 <- sum(R0^4)/M0

sigma <- sqrt(M2)       # zmiennosc

mu1 <- M3/(sigma^3)     ## skosnosc
mu2 <- M4/(sigma^4) -3  ## kurtoza

# VaR, VEV i MRM (w horyzoncie RHP)
VaR <-   sigma*sqrt(N)*(-1.96+0.474*mu1/sqrt(N) - 0.0687*mu2/N + 0.146*(mu1^2)/N) - 0.5*(sigma^2)*N
VEV <-   (sqrt(3.842 - 2*VaR) - 1.96)/ sqrt(RHP)

# Interpretacja VEV: porownaj do annualizowanego odchylenia standardowego
sigA = sigma*sqrt(Nyear)

MRM  <- as.numeric(cut(VEV, breaks = c(-Inf, 0.005, 0.05, 0.12, 0.2, 0.3, 0.8, Inf), labels = 1:7, right = FALSE))
MRM

#####################################################################################
# Scenariusze wedlug zalacznika IV Rozporzadzenia                                   #
#####################################################################################
V <- 1000   #  wartosc poczatkowa portfela

# Ustalenie liczby horyontow w zaleznosci od RHP zgodnie z Rozporzadeniem
if(RHP<=1){ Horyz <- RHP } else if(RHP<3){
    Horyz <- c(1,RHP)
} else {
    Horyz <- c(1,ceiling(RHP/2),RHP)
}
# Horyzont wyrazony w czestotliwosci danych
HoryzN   <- Horyz*Nyear

# Macierz z wynikami
Sc           <- matrix(NA,nrow=length(Horyz),ncol=5)
colnames(Sc) <- c("Lata", "Niekorzystny", "Umiarkowany", "Korzystny", "Skrajny")
Sc[,1]       <- Horyz

for(h in 1:length(HoryzN)){
    H0         <- HoryzN[h]
    RHP0       <- Horyz[h]
    q = c(0.1,0.5,0.9)  # kwantyle dla scenariuszy niekorzystnego, umiarkowanego, korzystnego
    for(ScN in 1:3){    # scenariusze niekorzystny, umiarkowany, korzystny
        PsiU    <- qnorm(q[ScN])
        temp    <- M1*H0 + sigma*sqrt(H0)*(PsiU + (PsiU^2-1)/6*mu1/sqrt(H0) + (PsiU^3-3*PsiU)/24*mu2/H0 - (2*PsiU^3-5*PsiU)/36*(mu1^2)/H0) - 0.5*(sigma^2)*H0
        Sc[h,ScN+1]  <- exp(temp)*V
    }

    # Scenariusz skrajny
    sigma_sstar <- SigStressed(r,RHP0)
    if(RHP0<=1){PsiU <- qnorm(0.01)} else {PsiU <- qnorm(0.05)}
    temp        <- sigma_sstar*sqrt(H0)*(PsiU + (PsiU^2-1)/6*mu1/sqrt(H0) + (PsiU^3-3*PsiU)/24*mu2/H0 - (2*PsiU^3-5*PsiU)/36*(mu1^2)/H0) - 0.5*(sigma_sstar^2)*H0
    Sc[h,5]     <- exp(temp)*V
}
  # Table
Sc <- data.frame(round(Sc,3))
Sc



#####################################################
#  Funkcja wyznacza stressed sigma                  #
#  T0       - horyzont w latach                     #
#  ret0     - stopy zwrotu                          #
#####################################################

SigStressed <- function(ret,RHP){

  Ret <- coredata(ret)

  if(RHP<=1){w <- 21}
  if(RHP >1){w <- 63}
  # dostosowanie dlugosci okna, aby bylo zgodne ze wzorem z punktu 10(c) Zalacznika IV
  w <- w+1

  # obliczenia szeregu rolowanych odchylen standardowych (punkt 10c, Zalcznik IV)
  # koniec kazego interwalu (od w do Tobs)
  seq <- sapply(w:(length(Ret)), function(t){
    retA       <- Ret[(t-w+1):t]                         # w+1 obserwacji
    retA       <- retA- mean(retA)                         # usuniecie sredniej
    Mw         <- length(retA)                             # liczba obserwacji
    sigma_w    <- sqrt(sum(retA^2)/Mw)
    return(sigma_w)}
  )
  sigma_sw <- zoo(seq, order.by = index(ret)[w:(length(ret))])

  # wartosci scenariusza wedlug wzoru  z punktu 11 zalacznika Rozporzadzenia
  if(RHP<=1){
    sigma_sstar <- as.numeric(quantile(sigma_sw, 0.99))
    PsiU        <- qnorm(0.01)
  } else {
    sigma_sstar <- as.numeric(quantile(sigma_sw, 0.90))
    PsiU        <- qnorm(0.05)
  }
  return(sigma_sstar)
}



